@@ -19,8 +19,8 @@ rho0 = qutip.coherent_dm(N, 2.0)
 n = qutip.mesolve(H, rho0, taus, c_ops, [a.dag() * a]).expect[0]
 
 # calculate the correlation function G1 and normalize with n to obtain g1
-G1 = qutip.correlation_2op_2t(H, rho0, None, taus, c_ops, a.dag(), a)
-g1 = G1 / np.sqrt(n[0] * n)
+G1 = qutip.correlation_2op_1t(H, rho0, taus, c_ops, a.dag(), a)
+g1 = np.array(G1) / np.sqrt(n[0] * np.array(n))
 
 plt.plot(taus, np.real(g1), 'b', lw=2)
 plt.plot(taus, n, 'r', lw=2)
