@@ -28,7 +28,7 @@ for state in states:
     # calculate the correlation function G2 and normalize with n(0)n(t) to
     # obtain g2
     G2 = qutip.correlation_3op_1t(H, rho0, taus, c_ops, a.dag(), a.dag()*a, a)
-    g2 = G2 / (n[0] * n)
+    g2 = np.array(G2) / (n[0] * np.array(n))
 
     ax.plot(taus, np.real(g2), label=state['label'], lw=2)
 
