@@ -7,36 +7,34 @@ eps0  = 1.0 * 2*np.pi
 A     = 0.25 * 2*np.pi
 omega = 1.0 * 2*np.pi
 T      = 2*np.pi / omega
-tlist  = np.linspace(0.0, 20 * T, 101)
+tlist  = np.linspace(0.0, 20 * T, 301)
 psi0   = qutip.basis(2,0)
 
 H0 = - delta/2.0 * qutip.sigmax() - eps0/2.0 * qutip.sigmaz()
 H1 = A/2.0 * qutip.sigmax()
 args = {'w': omega}
-H = [H0, [H1, lambda t,args: np.sin(args['w'] * t)]]
+H = [H0, [H1, lambda t, w: np.sin(w * t)]]
 
 # noise power spectrum
 gamma1 = 0.1
 def noise_spectrum(omega):
-    return 0.5 * gamma1 * omega/(2*np.pi)
-
-# find the floquet modes for the time-dependent hamiltonian        
-f_modes_0, f_energies = qutip.floquet_modes(H, T, args)
-
-# precalculate mode table
-f_modes_table_t = qutip.floquet_modes_table(
-    f_modes_0, f_energies, np.linspace(0, T, 500 + 1), H, T, args,
-)
+    return (omega>0) * 0.5 * gamma1 * omega/(2*np.pi)
 
 # solve the floquet-markov master equation
-output = qutip.fmmesolve(H, psi0, tlist, [qutip.sigmax()], [], [noise_spectrum], T, args)
+output = qutip.fmmesolve(
+    H, psi0, tlist, [qutip.sigmax()],
+    spectra_cb=[noise_spectrum], T=T,
+    args=args, options={"store_floquet_states": True}
+)
 
 # calculate expectation values in the computational basis
 p_ex = np.zeros(tlist.shape, dtype=np.complex128)
 for idx, t in enumerate(tlist):
-    f_modes_t = qutip.floquet_modes_t_lookup(f_modes_table_t, t, T)
-    f_states_t = qutip.floquet_states(f_modes_t, f_energies, t)
-    p_ex[idx] = qutip.expect(qutip.num(2), output.states[idx].transform(f_states_t, True))
+    f_coeff_t = output.floquet_states[idx]
+    psi_t = output.floquet_basis.from_floquet_basis(f_coeff_t, t)
+    # Alternatively
+    psi_t = output.states[idx]
+    p_ex[idx] = qutip.expect(qutip.num(2), psi_t)
 
 # For reference: calculate the same thing with mesolve
 output = qutip.mesolve(H, psi0, tlist,
