@@ -44,7 +44,7 @@ class TestOperator:
         iden = [qutip.qeye(n) for n in dimensions]
         for i, test in enumerate(test_operators):
             expected = qutip.tensor(iden[:i] + [a[i]] + iden[i+1:])
-            assert test == expected
+            assert test.data == expected.data
             assert test.dims == [dimensions, dimensions]
 
     def test_space_size_reduction(self, dimensions, n_excitations):
@@ -82,22 +82,23 @@ def test_fock_state(dimensions, n_excitations):
 def test_fock_state_error():
     with pytest.raises(ValueError) as e:
         state = qutip.enr_fock([2, 2, 2], 1, [1, 1, 1])
-    assert str(e.value).startswith("The state tuple ")
+    assert str(e.value).startswith("state tuple ")
 
 
 def _reference_dm(dimensions, n_excitations, nbars):
     """
-    Get the reference density matrix using `Qobj.eliminate_states` explicitly,
-    to compare to the direct ENR construction.
+    Get the reference density matrix explicitly, to compare to the direct ENR
+    construction.
     """
     if np.isscalar(nbars):
         nbars = [nbars] * len(dimensions)
-    out = qutip.tensor([qutip.thermal_dm(dimension, nbar)
-                        for dimension, nbar in zip(dimensions, nbars)])
-    eliminate = [
+    keep = np.array([
         i for i, state in enumerate(qutip.state_number_enumerate(dimensions))
-        if sum(state) > n_excitations]
-    out = out.eliminate_states(eliminate)
+        if sum(state) <= n_excitations
+    ])
+    dm = qutip.tensor([qutip.thermal_dm(dimension, nbar)
+                       for dimension, nbar in zip(dimensions, nbars)])
+    out = qutip.Qobj(dm.full()[keep[:, None], keep[None, :]])
     return out / out.tr()
 
 
@@ -115,6 +116,76 @@ def test_thermal_dm(dimensions, n_excitations, nbar_type):
     np.testing.assert_allclose(test_dm.full(), expect_dm.full(), atol=1e-12)
 
 
-def test_large_space():
-    # Check for #2249
-    qutip.enr_destroy([12]*12, 2)
+def test_mesolve_ENR():
+    # Ensure ENR states work with mesolve
+    # We compare the output to an exact truncation of the
+    # single-excitation Jaynes-Cummings model
+    eps = 2 * np.pi
+    omega_c = 2 * np.pi
+    g = 0.1 * omega_c
+    gam = 0.01 * omega_c
+    tlist = np.linspace(0, 20, 100)
+    N_cut = 2
+
+    sz = qutip.sigmaz() & qutip.qeye(N_cut)
+    sm = qutip.destroy(2).dag() & qutip.qeye(N_cut)
+    a = qutip.qeye(2) & qutip.destroy(N_cut)
+    H_JC = (0.5 * eps * sz + omega_c * a.dag()*a +
+            g * (a * sm.dag() + a.dag() * sm))
+    psi0 = qutip.basis(2, 0) & qutip.basis(N_cut, 0)
+    c_ops = [np.sqrt(gam) * a]
+
+    result_JC = qutip.mesolve(H_JC, psi0, tlist, c_ops, e_ops=[sz])
+
+    N_exc = 1
+    dims = [2, N_cut]
+    d = qutip.enr_destroy(dims, N_exc)
+    sz = 2*d[0].dag()*d[0]-1
+    b = d[0]
+    a = d[1]
+    psi0 = qutip.enr_fock(dims, N_exc, [1, 0])
+    H_enr = (eps * b.dag()*b + omega_c * a.dag() * a +
+             g * (b.dag() * a + a.dag() * b))
+    c_ops = [np.sqrt(gam) * a]
+
+    result_enr = qutip.mesolve(H_enr, psi0, tlist, c_ops, e_ops=[sz])
+
+    np.testing.assert_allclose(result_JC.expect[0],
+                               result_enr.expect[0], atol=1e-2)
+
+
+def test_steadystate_ENR():
+    # Ensure ENR states work with steadystate functions
+    # We compare the output to an exact truncation of the
+    # single-excitation Jaynes-Cummings model
+    eps = 2 * np.pi
+    omega_c = 2 * np.pi
+    g = 0.1 * omega_c
+    gam = 0.01 * omega_c
+    N_cut = 2
+
+    sz = qutip.sigmaz() & qutip.qeye(N_cut)
+    sm = qutip.destroy(2).dag() & qutip.qeye(N_cut)
+    a = qutip.qeye(2) & qutip.destroy(N_cut)
+    H_JC = (0.5 * eps * sz + omega_c * a.dag()*a +
+            g * (a * sm.dag() + a.dag() * sm))
+    c_ops = [np.sqrt(gam) * a]
+
+    result_JC = qutip.steadystate(H_JC, c_ops)
+    exp_sz_JC = qutip.expect(sz, result_JC)
+
+    N_exc = 1
+    dims = [2, N_cut]
+    d = qutip.enr_destroy(dims, N_exc)
+    sz = 2*d[0].dag()*d[0]-1
+    b = d[0]
+    a = d[1]
+    H_enr = (eps * b.dag()*b + omega_c * a.dag() * a +
+             g * (b.dag() * a + a.dag() * b))
+    c_ops = [np.sqrt(gam) * a]
+
+    result_enr = qutip.steadystate(H_enr, c_ops)
+    exp_sz_enr = qutip.expect(sz, result_enr)
+
+    np.testing.assert_allclose(exp_sz_JC,
+                               exp_sz_enr, atol=1e-2)
