@@ -16,9 +16,10 @@ from scipy.special import genlaguerre, binom, sph_harm, factorial
 
 import qutip
 from qutip import Qobj, ket2dm, jmat
-from qutip.parallel import parfor
-from qutip.cy.sparse_utils import _csr_get_diag
-from qutip.sparse import eigh
+from .solver.parallel import parallel_map
+from .utilities import clebsch
+from .core import data as _data
+from .core.data.eigen import eigh
 
 
 def wigner_transform(psi, j, fullparity, steps, slicearray):
@@ -67,8 +68,8 @@ def wigner_transform(psi, j, fullparity, steps, slicearray):
     if not (psi.type == 'ket' or psi.type == 'operator' or psi.type == 'bra'):
         raise TypeError('Input state is not a valid operator.')
 
-    if psi.type == 'ket' or psi.type == 'bra':
-        rho = ket2dm(psi)
+    if psi.isket or psi.isbra:
+        rho = psi.proj()
     else:
         rho = psi
 
@@ -93,8 +94,9 @@ def wigner_transform(psi, j, fullparity, steps, slicearray):
         pari = _parity(sun, j)
     for t in range(steps):
         for p in range(steps):
-            wigner[t, p] = np.real(np.trace(rho.data @ _kernelsu2(
-                theta[:, t], phi[:, p], N, j, pari, fullparity)))
+            kernel = _kernelsu2(theta[:, t], phi[:, p], N, j, pari, fullparity)
+            kernel = _data.dense.fast_from_numpy(kernel)
+            wigner[t, p] = _data.expect(rho.data, kernel).real
     return wigner
 
 
@@ -110,7 +112,7 @@ def _parity(N, j):
         foo = np.ones(mult)
         for n in np.arange(-j, j + 1, 1):
             for l in np.arange(0, mult, 1):
-                foo[l] = (2 * l + 1) * qutip.clebsch(j, l, j, n, 0, n)
+                foo[l] = (2 * l + 1) * clebsch(j, l, j, n, 0, n)
             matrix[np.int32(n + j), np.int32(n + j)] = np.sum(foo)
         return matrix / mult
 
@@ -159,7 +161,7 @@ def _angle_slice(slicearray, theta, phi):
     return theta, phi
 
 
-def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
+def wigner(psi, xvec, yvec=None, method='clenshaw', g=sqrt(2),
            sparse=False, parfor=False):
     """Wigner function for a state vector or density matrix at points
     `xvec + i * yvec`.
@@ -177,13 +179,13 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
         y-coordinates at which to calculate the Wigner function.  Does not
         apply to the 'fft' method.
 
-    g : float
+    g : float, default: sqrt(2)
         Scaling factor for `a = 0.5 * g * (x + iy)`, default `g = sqrt(2)`.
         The value of `g` is related to the value of `hbar` in the commutation
         relation `[x, y] = i * hbar` via `hbar=2/g^2` giving the default
         value `hbar=1`.
 
-    method : string {'clenshaw', 'iterative', 'laguerre', 'fft'}
+    method : string {'clenshaw', 'iterative', 'laguerre', 'fft'}, default: 'clenshaw'
         Select method 'clenshaw' 'iterative', 'laguerre', or 'fft', where 'clenshaw'
         and 'iterative' use an iterative method to evaluate the Wigner functions for density
         matrices :math:`|m><n|`, while 'laguerre' uses the Laguerre polynomials
@@ -195,12 +197,12 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
         dealing with density matrices that have a large number of excitations
         (>~50). 'clenshaw' is a fast and numerically stable method.
 
-    sparse : bool {False, True}
+    sparse : bool, optional
         Tells the default solver whether or not to keep the input density
         matrix in sparse format.  As the dimensions of the density matrix
         grow, setthing this flag can result in increased performance.
 
-    parfor : bool {False, True}
+    parfor : bool, optional
         Flag for calculating the Laguerre polynomial based Wigner function
         method='laguerre' in parallel using the parfor function.
 
@@ -235,8 +237,8 @@ def wigner(psi, xvec, yvec, method='clenshaw', g=sqrt(2),
     if method == 'fft':
         return _wigner_fourier(psi, xvec, g)
 
-    if psi.type == 'ket' or psi.type == 'bra':
-        rho = ket2dm(psi)
+    if psi.isket or psi.isbra:
+        rho = psi.proj()
     else:
         rho = psi
 
@@ -320,7 +322,7 @@ def _wigner_laguerre(rho, xvec, yvec, g, parallel):
         if parallel:
             iterator = (
                 (m, rho, A, B) for m in range(len(rho.data.indptr) - 1))
-            W1_out = parfor(_par_wig_eval, iterator)
+            W1_out = parallel_map(_par_wig_eval, iterator)
             W += sum(W1_out)
         else:
             for m in range(len(rho.data.indptr) - 1):
@@ -460,7 +462,7 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
 
     B = np.abs(A2)
     B *= B
-    w0 = (2*rho.data[0,-1])*np.ones_like(A2)
+    w0 = (2*rho[0, -1])*np.ones_like(A2)
     L = M-1
     #calculation of \sum_{L} c_L (2x)^L / \sqrt(L!)
     #using Horner's method
@@ -471,10 +473,11 @@ def _wigner_clenshaw(rho, xvec, yvec, g=sqrt(2), sparse=False):
             #here c_L = _wig_laguerre_val(L, B, np.diag(rho, L))
             w0 = _wig_laguerre_val(L, B, np.diag(rho, L)) + w0 * A2 * (L+1)**-0.5
     else:
+        # TODO: fix dispatch.
+        _rho = _data.to(_data.CSR, rho.data).as_scipy()
         while L > 0:
             L -= 1
-            diag = _csr_get_diag(rho.data.data,rho.data.indices,
-                                rho.data.indptr,L)
+            diag = _rho.diagonal(L)
             if L != 0:
                 diag *= 2
             #here c_L = _wig_laguerre_val(L, B, np.diag(rho, L))
@@ -529,7 +532,7 @@ def _qfunc_check_state(state: Qobj):
         state.isoper
         and state.dims[0] == state.dims[1]
         and state.isherm
-        and abs(state.tr() - 1) < qutip.settings.atol
+        and abs(state.tr() - 1) < qutip.settings.core['atol']
     )
     if not (state.isket or isdm):
         raise ValueError(
@@ -654,13 +657,13 @@ class QFunc:
     xvec, yvec : array_like
         x- and y-coordinates at which to calculate the Husimi-Q function.
 
-    g : float, default sqrt(2)
+    g : float, default: sqrt(2)
         Scaling factor for ``a = 0.5 * g * (x + iy)``.  The value of `g` is
         related to the value of `hbar` in the commutation relation
         :math:`[x,\,y] = i\hbar` via :math:`\hbar=2/g^2`, so the default
         corresponds to :math:`\hbar=1`.
 
-    memory : real, default 1024
+    memory : real, default: 1024
         Size in MB that may be used internally as workspace.  This class will
         raise ``MemoryError`` if subsequently passed a state of sufficiently
         large dimension that this bound would be exceeded.  In those cases, use
@@ -684,7 +687,7 @@ class QFunc:
     See Also
     --------
     :obj:`.qfunc` :
-        a single function version, which will involve computing several
+        A single function version, which will involve computing several
         quantities multiple times in order to use less memory.
     """
 
@@ -788,13 +791,13 @@ def qfunc(
     xvec, yvec : array_like
         x- and y-coordinates at which to calculate the Husimi-Q function.
 
-    g : float, default sqrt(2)
+    g : float, default: sqrt(2)
         Scaling factor for ``a = 0.5 * g * (x + iy)``.  The value of `g` is
         related to the value of :math:`\hbar` in the commutation relation
         :math:`[x,\,y] = i\hbar` via :math:`\hbar=2/g^2`, so the default
         corresponds to :math:`\hbar=1`.
 
-    precompute_memory : real, default 1024
+    precompute_memory : real, default: 1024
         Size in MB that may be used during calculations as working space when
         dealing with density-matrix inputs.  This is ignored for state-vector
         inputs.  The bound is not quite exact due to other, order-of-magnitude
@@ -803,7 +806,7 @@ def qfunc(
         that is used for single kets, set ``precompute_memory=None``.
 
     Returns
-    --------
+    -------
     ndarray
         Values representing the Husimi-Q function calculated over the specified
         range ``[xvec, yvec]``.
@@ -884,11 +887,8 @@ def spin_q_function(rho, theta, phi):
 
     """
 
-    if rho.type == 'bra':
-        rho = rho.dag()
-
-    if rho.type == 'ket':
-        rho = ket2dm(rho)
+    if rho.isket or rho.isbra:
+        rho = rho.proj()
 
     J = rho.shape[0]
     j = (J - 1) / 2
@@ -896,18 +896,19 @@ def spin_q_function(rho, theta, phi):
     THETA, PHI = meshgrid(theta, phi)
 
     Q = np.zeros_like(THETA, dtype=complex)
+    data = rho.full()
 
     for m1 in arange(-j, j + 1):
         Q += binom(2 * j, j + m1) * cos(THETA / 2) ** (2 * (j + m1)) * \
              sin(THETA / 2) ** (2 * (j - m1)) * \
-             rho.data[int(j - m1), int(j - m1)]
+             data[int(j - m1), int(j - m1)]
 
         for m2 in arange(m1 + 1, j + 1):
             Q += (sqrt(binom(2 * j, j + m1)) * sqrt(binom(2 * j, j + m2)) *
                   cos(THETA / 2) ** (2 * j + m1 + m2) *
                   sin(THETA / 2) ** (2 * j - m1 - m2)) * \
-             (exp(1j * (m1 - m2) * PHI) * rho.data[int(j - m1), int(j - m2)] +
-              exp(1j * (m2 - m1) * PHI) * rho.data[int(j - m2), int(j - m1)])
+             (exp(1j * (m1 - m2) * PHI) * data[int(j - m1), int(j - m2)] +
+              exp(1j * (m2 - m1) * PHI) * data[int(j - m2), int(j - m1)])
 
     return Q.real, THETA, PHI
 
@@ -935,13 +936,14 @@ def _rho_kq(rho, j, k, q):
     """
 
     v = 0j
+    data = rho.full()
     for m1 in arange(-j, j+1):
         for m2 in arange(-j, j+1):
             v += (
                     (-1) ** (2 * j - k - m1 - m2)
                     * np.sqrt((2 * k + 1) / (2 * j + 1))
-                    * qutip.clebsch(j, k, j, -m1, q, -m2)
-                    * rho.data[int(j - m1), int(j - m2)]
+                    * clebsch(j, k, j, -m1, q, -m2)
+                    * data[int(j - m1), int(j - m2)]
             )
     return v
 
@@ -983,12 +985,8 @@ def spin_wigner(rho, theta, phi):
     taken from Wikipedia (https://en.wikipedia.org/wiki/3-j_symbol)
 
     """
-
-    if rho.type == 'bra':
-        rho = rho.dag()
-
-    if rho.type == 'ket':
-        rho = ket2dm(rho)
+    if rho.isket or rho.isbra:
+        rho = rho.proj()
 
     J = rho.shape[0]
     j = (J - 1) / 2
